/*
 * NET		An implementation of the SOCKET network access protocol.
 *		This module implements the PPP protocol for the kernel-
 *		based devices like TTY.  It interfaces between a raw TTY,
 *		and the kernel's NET protocol layers (via DDI).
 *
 *		Various functions.
 *
 * Version:	@(#)utils.c	0.2.5	05/12/94
 *
 * Authors:	Michael Callahan <callahan@maths.ox.ac.uk>
 *		Fred N. van Kempen, <waltje@uwalt.nl.mugnet.org>
 *
 *		This program is  free  software; you  can redistribute  it
 *		and/or modify it under the terms of the GNU General Public
 *		License  as  published by the  Free  Software  Foundation;
 *		either version 2  of the License, or (at  your option) any
 *		later version.
 */
#include <asm/segment.h>
#include <asm/system.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/interrupt.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/socket.h>
#include <linux/timer.h>
#include <linux/ddi.h>
#include <linux/skbuff.h>
#include <linux/in.h>
#include <linux/in_systm.h>
#include <linux/if.h>
#include <linux/if_ppp.h>
#include <linux/ip.h>
#include <linux/errno.h>
#include <linux/stat.h>
#include <linux/tty.h>
#include <stdarg.h>
#include "ppp.h"
#include "global.h"


int		ppp_debug = 0;			/* PPP debugging flag	*/
int		ppp_debug_netpackets = 0;
#if CHECK_CHARS
unsigned long paritytab[8] = {
  0x96696996, 0x69969669, 0x69969669, 0x96696996,
  0x69969669, 0x96696996, 0x96696996, 0x69969669
};
#endif


#ifdef PPP_DEBUG
void
ppp_dprintf(char *fmt, ...)
{
  char buff[256];
  va_list args;
  extern int vsprintf(char *buf, const char *fmt, va_list args);

  if (ppp_debug == 0) return;

  va_start(args, fmt);
  vsprintf(buff, fmt, args);
  va_end(args);
  printk("%s", buff);
}


static inline void
ppp_print_hex(register char *out, char *in, int count)
{
  static char hex[] = "0123456789ABCDEF";
  register unsigned char next_ch;

  while (count-- > 0) {
	next_ch = (unsigned char) get_fs_byte(in);

	*out++ = hex[(next_ch >> 4) & 0x0F];
	*out++ = hex[next_ch & 0x0F];
	in++;
  }
}


static inline void
ppp_print_char(register char *out, char *in, int count)
{
  register unsigned char next_ch;

  while (count-- > 0) {
	next_ch = (unsigned char) get_fs_byte(in);

	if (next_ch < 0x20 || next_ch > 0x7e) *out++ = '.';
	  else {
		*out++ = *in;
		/* printk/syslogd has a bug !! */
		if (next_ch == '%') *out++ = '%';
	}
	in++;
  }
  *out = '\0';
}


void
ppp_print_buffer(const char *name, char *buf, int count, int seg)
{
  char line [44];
  int  old_fs = get_fs();

  set_fs(seg);

  if (name != (char *) NULL)
	DPRINTF(("PPP: %s, count = %d\n", name, count));

  while (count > 8) {
	memset(line, ' ', sizeof(line));
	ppp_print_hex(line, buf, 8);
	ppp_print_char(&line[8 * 3], buf, 8);
	DPRINTF(("%s\n", line));
	count -= 8;
	buf += 8;
  }

  if (count > 0) {
	memset(line, ' ', sizeof(line));
	ppp_print_hex(line, buf, count);
	ppp_print_char(&line[8 * 3], buf, count);
	DPRINTF(("%s\n", line));
  }

  set_fs(old_fs);
}
#endif	/* PPP_DEBUG */


int
ppp_dbg_ioctl(unsigned long arg, int level)
{
  int val;

  val = get_fs_long((int *) arg);
  switch(val) {
	case 0:	/* OFF */
		ppp_debug = 0;
		break;

	case 1:	/* ON */
		ppp_debug = level;
		break;
	default:
		if (val >= 128) {
			ppp_debug = 0;
			return(0);
		}
		ppp_debug = val;
		break;
  }

  return(0);
}
